# -*- encoding: utf-8 -*-
# (C) Copyright 2008 Tarek Ziadé <tarek@ziade.org>
""" 
Core module. contains processor.
"""
import socket
import os

from multiprocessing import Pool
from multiprocessing import TimeoutError
from multiprocessing import Queue

from atomisator.main.config import log
from atomisator.main.config import dotlog
from atomisator.main.config import AtomisatorConfig

from atomisator.main import FILTERS
from atomisator.main import OUTPUTS
from atomisator.main import ENHANCERS
from atomisator.main import READERS
from atomisator.main import load_plugin

from atomisator.db.session import create_session
from atomisator.db.core import create_entry
from atomisator.db.core import get_entries
from atomisator.db.core import purge_entries

#
# internal APIs called by multiprocessing
#
def _select_enhancers(enhancers_selected):
    """Gets the selected enhancers."""
    res = []
    for name, args in enhancers_selected:
        if name in ENHANCERS:
            res.append((ENHANCERS[name], args))
    return res

def _select_outputs(outputs_selected):
    """Gets the selected outputs."""
    res = []
    for name, args in outputs_selected:
        if name in OUTPUTS:
            res.append((OUTPUTS[name], args))
    return res

def _apply_filters(entry, entries, selected_filters):
    """Applies all selected filters to an entry"""
    for filterer, args in selected_filters:
        entry = filterer(entry, entries, *args)
        if entry is None:
            return None
    return entry

def _process_source(reader_name, reader, reader_args):
    """processing one source. callable through 
    a process"""
    try:
        log('\tRetrieving from %s - %s' %  (reader_name, str(reader_args)))
        return reader(*reader_args)
    except TimeoutError:
        log('\tTIMEOUT on %s - %s' % (reader_name, str(reader_args)))
        return []

def _enhance(entry, selected_enhancers):
    """Enhances an entry"""
    for enhancer, args in selected_enhancers:
        dotlog('.')
        entry = enhancer(entry, *args)
        if entry is None:
            return None
    return entry

def _prepare_enhancer(enhancer, entries):
    """prepare the enhancer"""
    if hasattr(enhancer, 'prepare'):
        dotlog('.')
        enhancer.prepare(entries)
    return enhancer

#
# class that processes data
#
class DataProcessor(object):
    """Atomisator processor
    
    Knows how to load datas and generate outputs.
    """

    def __init__(self, conf):
        self.parser = AtomisatorConfig(conf)
        self.existing_entries = []
        self.filter_chain = None
        create_session(self.parser.database)

    def load_data(self):
        """Fetches data"""
        log('Loading existing data.')
        old_timeout = socket.getdefaulttimeout()
        socket.setdefaulttimeout(self.parser.timeout)
        try:
            self._load_data()
        finally:
            socket.setdefaulttimeout(old_timeout)
   
    def _cleanup_entries(self):
        """removes old entries"""
        purge_entries(self.parser.max_age)

    def _load_data(self):
        """Loads the data"""
        # remove the temp db if it exists
        if os.path.exists('_temp_.db'):
            os.remove('_temp_.db')
       
        # cleanup old entries
        log('\tRemoving old entries.')
        self._cleanup_entries()

        # initial entries, see if this call is optimal
        log('\tReading existing entries.')
        self.existing_entries = get_entries().all()

        # building filtering chain once.
        filter_names = [f[0] for f in self.parser.filters]
        log('Loading filters plugins : %s' % ', '.join(filter_names))
        self.filter_chain = set([(load_plugin(name, FILTERS), args) 
                                 for name, args in self.parser.filters 
                                 if name in FILTERS])
        
        # building source chain once.
        source_names = [s[0] for s in self.parser.sources]
        log('Loading source plugins : %s' % ', '.join(source_names))
        sources = [(reader_name, load_plugin(reader_name, READERS), args) 
                   for reader_name, args in self.parser.sources]

        processes = self.parser.processes
        log('Reading sources.')

        if processes == 1:
            log('\tWorking in the same process')
            for source in sources:
                entries = _process_source(*source)
                self._process_entries(entries)
        else:    
            # creating a processing pool  
            pool = Pool(self.parser.processes)
            for source in sources:
                log('\tLaunching worker for %s - %s' % (source[0], source[-1]))
                pool.apply_async(_process_source, source, 
                                 callback=self._process_entries)

            pool.close()
            pool.join()
        dotlog('\n')

    def _process_entries(self, entries):
        """callback called by the worker"""
        # now lets apply filters, then store entries
        psession = create_session(self.parser.database, 
                                  global_=False) 
        for entry in entries:
            dotlog('.')
            entry = _apply_filters(entry, self.existing_entries, 
                                   self.filter_chain)
            if entry is None:
                continue
            new_entry = create_entry(entry, commit=False,
                                     session=psession)
            self.existing_entries.append(new_entry[1])
        psession.commit()

    def generate_data(self):
        """Generates the output"""
        selected_enhancers = _select_enhancers(self.parser.enhancers)
        selected_outputs = _select_outputs(self.parser.outputs)
        processes = self.parser.processes

        # XXX TODO: limit the size of the data 
        # processed, by number of items, or by date
        entries = get_entries().all()
        
        if selected_enhancers != []:
            # Enhancement is a two-phase process.
            # 1. Preparing entries for enhancement
            log('Preparing enhancers.')
            if processes == 1:
                class Wrap(object):
                    def __init__(self, res):
                        self.res = res
                    def get(self):
                        return self.res
        
                results = [Wrap(e) for e in 
                           [_prepare_enhancer(enhancer, entries)
                           for enhancer, args in selected_enhancers]]
            else:    
                pool = Pool(self.parser.processes)
                results = [pool.apply_async(_prepare_enhancer, 
                                            (enhancer, entries))
                       for enhancer, args in selected_enhancers]
                pool.close()
                pool.join()

            # pushing back into the ENHANCERS list (not the same process)
            # and selected_enhancers
            # XXX maybe the multiprocess has something cleaner for this
            for result in results:
                enhancer = result.get()
                enhancer_type = type(enhancer)
                for name, ob in ENHANCERS.items():
                    if type(ob) == enhancer_type:
                        ENHANCERS[name] = enhancer

                def _replace(s, args, new):
                    if type(s) == type(new):
                        s = new
                    return s, args

                selected_enhancers = [_replace(s, args, enhancer) 
                                      for s, args in selected_enhancers]

            dotlog('\n')

            # 2. Now enhancing them
            log('Enhancing: %s' % ', '.join([e for e, args 
                                            in self.parser.enhancers]))

            if processes == 1:
                results = [e for e in [_enhance(entry, selected_enhancers) 
                           for entry in entries] if e is not None] 
            else:
                # creating a processing pool
                pool = Pool(self.parser.processes)
                results = [pool.apply_async(_enhance, 
                           (entry, selected_enhancers))
                           for entry in entries]

                pool.close()
                pool.join()
                entries = [r for r in [result.get() for result in results] 
                           if r is not None]
            dotlog('\n')

        if selected_outputs != []:
            log('Writing outputs.')
            log('Rendering: %s' % ', '.join([output[0] for output 
                                             in self.parser.outputs]))
            for output, args in selected_outputs:
                dotlog('.')
                output(entries, args)
            dotlog('\n')

            log('Output ready.')




import os
from nose.tools import *

from atomisator.main.config import AtomisatorConfig
from atomisator.main.config import generate_config 
from atomisator.main.config import CONF_TMPL
from atomisator.main import PROCESSES

cfg = os.path.join(os.path.dirname(__file__), 'test.cfg')
cfg2 = os.path.join(os.path.dirname(__file__), 'test2.cfg')

def gen_init():
    if os.path.exists('here.cfg'):
        os.remove('here.cfg')

@with_setup(setup=gen_init, teardown=gen_init)
def test_generation():
    # Let's try a generation
    generate_config('here.cfg')
    assert_equals(open('here.cfg').read(), CONF_TMPL)


def test_config():
    parser = AtomisatorConfig(cfg)
    s = parser.sources 
    waited = [('rss', ('gdigg.xml',)), ('rss', ('gtarek.xml',)), 
              ('rss', ('gpp.xml',)), ('rss', ('gdigg.xml',)), 
              ('rss', ('gtarek.xml',)), ('rss', ('gpp.xml',))]

    assert_equals(s, waited)
    parser.sources = (('rss', ('ok.xml',)),)
    assert_equals(parser.sources, [('rss', ('ok.xml',))])

    assert_equals(parser.database, 'sqlite:///gatomisator.db') 
    parser.database = 'sqlite://here'
    assert_equals(parser.database, 'sqlite://here')    

    assert_equals(parser.timeout, 5.0) 
    parser.timeout = 7.0
    assert_equals(parser.timeout, 7.0)

    assert_equals(parser.store_entries, True) 
    parser.store_entries = False
    assert_equals(parser.store_entries, False)

    assert_equals(parser.max_age, 30) 
    parser.max_age = 35
    assert_equals(parser.max_age, 35)

    old = open(cfg).read()
    parser.write()
    new = open(cfg).read()
    assert new != old

    open(cfg, 'w').write(old)

    assert_equals(parser.processes, PROCESSES)
    parser.processes = 1
    assert_equals(parser.processes, 1)


def test_quotes():
    parser = AtomisatorConfig(cfg)
    rss = ('rss', ('output.xml',  'http://link.xml', 
             'This is the output', 
             'This is the description'))
    parser.outputs = [rss]

    assert_equals(parser.outputs, [rss])

def test_defaults():
    parser = AtomisatorConfig(cfg2)
    assert_equals(parser.enhancers, [])
    assert_equals(parser.filters, [])
    assert_equals(parser.outputs, [])





# -*- encoding: utf-8 -*-
# (C) Copyright 2008 Tarek Ziadé <tarek@ziade.org>
"""Main module, loads entry points
"""
__version__ = '0.5.7'

from setuptools.package_index import iter_entry_points
try:
    from multiprocessing import cpu_count
except ImportError:
    # maybe not loaded yet
    def cpu_count():
        return 1

def _load_entry_point(name):
    """Loads an atomisator entry point."""
    entry_points = iter_entry_points(name)
    return dict([(e.name, e.load()()) for e in entry_points])

FILTERS = _load_entry_point('atomisator.filters')
OUTPUTS = _load_entry_point('atomisator.outputs')
ENHANCERS = _load_entry_point('atomisator.enhancers')
READERS = _load_entry_point('atomisator.readers')

def load_plugin(name, kind):
    """returns a registered plugin.
    
    Will raise an error if the plugin does not exists"""
    if name not in kind:
        raise ValueError('Could not load %s plugin.' % name)
    return kind[name]

# we'll use two processes per CPU
PROCESSES = cpu_count() * 2




import sys
import os

from nose.tools import *

from atomisator.main.commands import atomisator, _parse_options
from atomisator.main.config import AtomisatorConfig
from atomisator.main.config import ConfigurationError
from atomisator.main.core import DataProcessor
from atomisator.main.commands import ENHANCERS, FILTERS, READERS

from atomisator.main.tests.base import set_conf, tear_conf

saved = None

def setup():
    global saved
    saved = sys.argv

def teardown():
    sys.argv = saved

@with_setup(setup, teardown)
def test_options():
    # making sure we know how to parse options
    sys.argv = ['atomisator']
    options = _parse_options()
    assert_equals(options.read, True)
    assert_equals(options.generate, True)
    assert_equals(options.create, None)
    assert_equals(options.config, 'atomisator.cfg')

    sys.argv = ['atomisator', '-c', 'myfile.cfg']
    options = _parse_options()
    assert_equals(options.create, 'myfile.cfg')
    assert_equals(options.read, False)
    assert_equals(options.generate, False)

    sys.argv = ['atomisator', '-r', '-g', '-f', 'here.cfg']
    options = _parse_options()
    assert_equals(options.create, None)
    assert_equals(options.read, True)
    assert_equals(options.generate, True)
    assert_equals(options.config, 'here.cfg')

    sys.argv = ['atomisator', 'here.cfg']
    options = _parse_options()
    assert_equals(options.create, None)
    assert_equals(options.read, True)
    assert_equals(options.generate, True)
    assert_equals(options.config, 'here.cfg')

    sys.argv = ['atomisator', 'here.cfg', '-g']
    options = _parse_options()
    assert_equals(options.create, None)
    assert_equals(options.read, False)
    assert_equals(options.generate, True)
    assert_equals(options.config, 'here.cfg')

test_dir = os.path.dirname(__file__)
test_conf = os.path.join(test_dir, 'atomisator.cfg')

@with_setup(set_conf, tear_conf)
def test_load_data():

    DataProcessor(test_conf).load_data()

@with_setup(set_conf, tear_conf)
def test_generate_data():
    DataProcessor(test_conf).generate_data()

@with_setup(set_conf, tear_conf)
def test_config():
    parser = AtomisatorConfig(test_conf)
    sources = parser.sources
    sources.sort()
    dir = os.path.dirname(test_conf)
    wanted = [('rss', ('%s/digg.xml' % dir,)), 
              ('rss', ('%s/digg.xml' % dir,)), 
              ('rss', ('%s/pp.xml' % dir,)), 
              ('rss', ('%s/pp.xml' % dir,)), 
              ('rss', ('%s/tarek.xml' % dir,)),
              ('rss', ('%s/tarek.xml' % dir,))]


    assert_equals(sources, wanted)
    wanted = os.path.join(dir, 'atomisator.db')
    assert_equals(parser.database, 'sqlite:///%s' % wanted)

    # getting readers
    assert_equals(parser.get_reader('xxx'), None)
    assert_equals(parser.get_reader('xml'), 
                  'atomisator.reader.xml')

    f = [f[0] for f in parser.filters]
    f.sort()
    assert_equals(f, ['autotags', 'buzzwords', 'doublons', 'spam', 'stopwords'])

def test_get_readers():
    # see if we get the rss and atom plugin
    from atomisator.parser import Parser
   
    assert 'xxxx' not in READERS
    assert isinstance(READERS['rss'], Parser)

#def test_filters():
#    assert 'doublons' in FILTERS

def test_enhancers():
    assert 'digg' in ENHANCERS

@with_setup(set_conf, tear_conf)
def test_no_storage():
    proc = DataProcessor(test_conf)
    proc.parser.store_entries = False
   
    proc.load_data()
    proc.generate_data()   
       
    # as usual but in memory
    assert_equals(len(proc.existing_entries), 225)

@with_setup(set_conf, tear_conf)
def test_mono_proc():
    proc = DataProcessor(test_conf)
    proc.parser.processes = 1
    proc.load_data()




# -*- encoding: utf-8 -*-
# (C) Copyright 2008 Tarek Ziadé <tarek@ziade.org>
"""
Custom config parser for Atomisator, and logging utilities.
"""
import os
from os.path import dirname
import sys
from os.path import join
from ConfigParser import ConfigParser
import re
import logging

from atomisator.main import PROCESSES

# XXX make logging level configurable
logger = logging.getLogger("atomisator")
formatter = logging.Formatter()
handler = logging.StreamHandler(sys.stdout)
handler.setFormatter(formatter)
logger.addHandler(handler)
logger.setLevel(logging.INFO)

class ConfigurationError(Exception):
    pass

# logging
def log(msg):
    """Shortcut to logger."""
    logger.info(msg)

# see how this can be done with logging
def dotlog(msg):    
    """Flushes to stdout."""
    sys.stdout.write(msg)
    sys.stdout.flush()

DEFAULT_CONFIG_FILE = 'atomisator.cfg'
CONF_TMPL = join(dirname(__file__), 
                 'atomisator.cfg_tmpl')
CONF_TMPL = open(CONF_TMPL).read()

def generate_config(path):
    """Creates a default config file."""
    if os.path.exists(path):
        raise ValueError('%s already exists.' % path)
    file_ = open(path, 'w')
    try:
        file_.write(CONF_TMPL)
    finally:
        file_.close()
    logger.info('Default config generated at "%s."' % path)

class AtomisatorConfig(object):
    """Custom config parser
    """

    def __init__(self, cfg=None):
        self._parser = ConfigParser()
        if cfg is None:
            cfg = DEFAULT_CONFIG_FILE
        if not os.path.exists(cfg):
            raise ValueError('Could not read configuration (%s)' % cfg)
        self._file = cfg
        self._parser.read([cfg])

    #
    # basic APIs
    #
    def _get_simple_field(self, field, default=None):
        """Returns an option located in the `atomisator` section."""
        if not self._parser.has_option('atomisator', field):
            return default
        return self._parser.get('atomisator', field).strip()
    
    def _set_simple_field(self, field, value):
        """Set an option located in the `atomisator` section."""
        self._parser.set('atomisator', field, value)    

    def _set_multiline_value(self, name, value):
        """Set a multiline option located in the `atomisator` section."""
        def _quote(value):
            if ' ' in value:
                value = value.replace('"', "'")
                return '"%s"' % value
            return value

        def _line(entry):
            main = entry[0]
            params = [_quote(e) for e in entry[1]]
            return '%s %s' % (main, ' '.join(params))
        
        values = '\n'.join([_line(v) for v in value])
        self._parser.set('atomisator', name, values)

    def _get_multiline_value(self, name, default=None):
        """Returns a multiline option located in the `atomisator` section."""
        if not self._parser.has_option('atomisator', name):
            return default is None and None or []
        lines = self._parser.get('atomisator', name).split('\n')
        # crappy pattern matching
        def _rep(match):
            return match.groups()[0].replace(' ', ':::')
        def _args(line):
            line = re.sub(r'"(.*?)"', _rep, line)
            line = [element.replace(':::', ' ') 
                    for element in line.split() if element != '']
            return line[0].strip(), tuple([el.strip() 
                                           for el in line[1:]])
        return [_args(line) for line in lines if line.strip() != '']

    # 
    # properties and public APIs
    #
    def write(self):
        """ Saves the configuration.
        
        Writes the configuration into the file provided at initialization."""
        file_ = open(self._file, 'w')
        try:
            self._parser.write(file_)
        except:
            file_.close()

    def get_reader(self, name):
        """Returns the readers."""
        if not self._parser.has_section('readers'):
            return None
        if not self._parser.has_option('readers', name):
            return None
        return self._parser.get('readers', name)

    def _get_sources(self):
        return self._get_multiline_value('sources')
    def _set_sources(self, sources):
        self._set_multiline_value('sources', sources)
    sources = property(_get_sources, _set_sources)

    def _get_outputs(self):
        return self._get_multiline_value('outputs') 
    def _set_outputs(self, outputs):
        self._set_multiline_value('outputs', outputs)
    outputs = property(_get_outputs, _set_outputs)

    def _get_filters(self):
        return self._get_multiline_value('filters') 
    def _set_filters(self, filters):
        self._set_multiline_value('filters', filters)
    filters = property(_get_filters, _set_filters)

    def _get_enhancers(self):
        return self._get_multiline_value('enhancers')
    def _set_enhancers(self, value):
        self._set_multiline_value('enhancers', value)
    enhancers = property(_get_enhancers, _set_enhancers)

    def _get_database(self):
        return self._get_simple_field('database', 'sqlite:///_temp_.db')
    def _set_database(self, value):
        self._set_simple_field('database', value)
    database = property(_get_database, _set_database)

    def _get_timeout(self):
        return float(self._get_simple_field('timeout', '5'))
    def _set_timeout(self, value):
        self._set_simple_field('timeout', str(value))
    timeout = property(_get_timeout, _set_timeout)

    def _get_store_entries(self):
        value = self._get_simple_field('store-entries', 'true')
        value = value.lower()
        return value in ('true', '1')
    def _set_store_entries(self, value):
        value = value and 'true' or 'false'
        self._set_simple_field('store-entries', value)
    store_entries = property(_get_store_entries, _set_store_entries)

    def _get_max_age(self):
        return int(self._get_simple_field('max-age', '30'))
    def _set_max_age(self, value):
        self._set_simple_field('max-age', str(value))
    max_age = property(_get_max_age, _set_max_age)

    def _get_processes(self):
        return int(self._get_simple_field('processes', PROCESSES))
    def _set_processes(self, value):
        self._set_simple_field('processes', str(value))
    processes = property(_get_processes, _set_processes)





import os

test_dir = os.path.dirname(__file__)
package_dir = os.path.split(test_dir)[0]
test_conf = os.path.join(test_dir, 'atomisator.cfg')

def set_conf():
    template = open(os.path.join(test_dir, 'atomisator.cfg_tmpl')).read()
    cfg = template % {'test_dir': test_dir}
    f = open(test_conf, 'w')
    f.write(cfg)
    f.close()

def tear_conf():
    if os.path.exists(test_conf):
        os.remove(test_conf)
    xml = os.path.join(test_dir, 'atomisator.xml')
    if os.path.exists(xml):
        os.remove(xml)





import sys

from nose.tools import *

from atomisator.main.tests.base import set_conf, tear_conf
from atomisator.main.tests.base import test_conf 
from atomisator.main.commands import atomisator 
from atomisator.main.config import AtomisatorConfig

sys_argv = None

def set_more_conf():
    set_conf()
    parser = AtomisatorConfig(test_conf)
    parser.store_entries = False
    parser.write()
    global sys_argv 
    sys_argv = sys.argv 
    sys.argv = ['atomisator', test_conf]

def more_tear_conf():
    sys.argv = sys_argv 
    tear_conf()
 
@with_setup(set_more_conf, more_tear_conf)
def test_atomisator_no_storage():
    atomisator()




from setuptools import setup, find_packages
import os

from atomisator.main import __version__ as VERSION

long_description = open("README.txt").read()
classifiers=[
        "Programming Language :: Python",
        "Topic :: Software Development :: Libraries :: Python Modules",
        ]

entry_points = {
    "console_scripts": [
        "atomisator = atomisator.main.commands:atomisator"
    ]
}

setup(name='atomisator.main',
      version=VERSION,
      description="Expert Python Programming - Main package for Atomisator app",
      long_description=long_description,
      classifiers=classifiers,
      keywords='',
      author='Tarek Ziade',
      author_email='tarek@ziade.org',
      url='http://atomisator.ziade.org',
      license='GPL',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['atomisator'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'setuptools',
          'multiprocessing',
          'atomisator.db>=0.3.4',
          'atomisator.feed>=0.3.2',
          # default plugins
          'atomisator.parser>=0.2.6',
          'atomisator.readers>=0.1.1',
          # default filters
          'atomisator.filters>=0.1.9',
          # default enhancers
          'atomisator.enhancers>=0.1.1',
          # default outputs
          'atomisator.outputs>=0.1.2'
      ],
      tests_require=['nose'],
      test_suite='atomisator.main.tests.test_docs.test_suite',
      entry_points=entry_points
      )




import unittest
import doctest
import os

# you can add variables here, that will be available 
# in the doctests

test_dir = os.path.dirname(__file__)
package_dir = os.path.split(test_dir)[0]
test_conf = os.path.join(test_dir, 'atomisator.cfg')

def setUp(test):
    template = open(os.path.join(test_dir, 'atomisator.cfg_tmpl')).read()
    cfg = template % {'test_dir': test_dir}
    f = open(test_conf, 'w')
    f.write(cfg)
    f.close()

def tearDown(test):
    if os.path.exists(test_conf):
        os.remove(test_conf)

def doc_suite(test_dir):
    """Returns a test suite, based on doctests found in /doctest."""
    suite = []
    globs = globals()

    flags = (doctest.ELLIPSIS | doctest.NORMALIZE_WHITESPACE |
             doctest.REPORT_ONLY_FIRST_FAILURE)

    doctest_dir = os.path.join(package_dir, 'docs')

    # filtering files on extension
    docs = [os.path.join(doctest_dir, doc) for doc in
            os.listdir(doctest_dir) if doc.endswith('.txt')]

    for test in docs:
        suite.append(doctest.DocFileSuite(test, optionflags=flags, 
                                          globs=globs,
                                          module_relative=False,
                                          setUp=setUp, tearDown=tearDown))
    
    return unittest.TestSuite(suite)

def test_suite():
    """Returns the test suite."""
    return doc_suite(test_dir)

if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')




# -*- encoding: utf-8 -*-
# (C) Copyright 2008 Tarek Ziadé <tarek@ziade.org>
"""
Provides high-level commands and the options parser.
"""
import sys
from optparse import OptionParser

from atomisator.main.core import DataProcessor
from atomisator.main.config import generate_config

from atomisator.main import __version__ as VERSION
from atomisator.main import FILTERS
from atomisator.main import OUTPUTS
from atomisator.main import ENHANCERS
from atomisator.main import READERS

def _parse_options():
    """parses Atomisator args options"""
    parser = OptionParser(usage='usage: %prog [options]',
                          version='%%prog %s' % VERSION)

    parser.add_option("-c", "--create-config", dest="create",
                      action="store",
                      help="Creates a default config file.",
                      metavar="CONFIG_FILE") 

    parser.add_option("-r", "--read", dest="read",
                      action="store_true",
                      help="Reads sources.", default=False)
 
    parser.add_option("-l", "--list-filters", dest="filters",
                      action="store_true",
                      help="List all filters.", default=False)
   
    parser.add_option("-p", "--list-readers", dest="readers",
                      action="store_true",
                      help="List all reader plugins.", default=False)

    parser.add_option("-e", "--list-enhancers", dest="enhancers",
                      action="store_true",
                      help="List all enhancer plugins.", default=False)

    parser.add_option("-o", "--list-outputs", dest="outputs",
                      action="store_true",
                      help="List all output plugins.", default=False)

    parser.add_option("-g", "--generate", dest="generate",
                      action="store_true",
                      help="Generates feed.", default=False)
        
    parser.add_option("-f", "--config-file", dest="config",
                      help="Points to the configuration file.",
                      metavar="CONFIG_FILE") 
    
    options, args = parser.parse_args()

    if options.create is not None:
        options.read = False
        options.generate = False
    else:
        if options.config is None:
            if len(args) > 0:
                options.config = args[0]
            else:
                options.config = 'atomisator.cfg'

        if options.read is False and options.generate is False:
            options.read = True
            options.generate = True

    return options

def list_readers():
    """List out readers plugins."""
    for reader_name, reader in READERS.items():
        print reader_name
        doc = reader.__doc__
        if doc is not None:
            print doc

def list_outputs():
    """List out output plugins."""
    for output_name, output in OUTPUTS.items():
        print '%s: %s' % (output_name, output.__doc__)
           
def list_filters():
    """List out filter plugins."""
    for filter_name, filter_ in FILTERS.items():
        print '%s: %s' % (filter_name, filter_.__doc__)

def list_enhancers():
    """List out enhancers plugins."""
    for enhancer_name, enhancer in ENHANCERS.items():
        print '%s: %s' % (enhancer_name, enhancer.__doc__)

def atomisator():
    """Main function."""
    options = _parse_options()
    if options.filters:
        list_filters()
        sys.exit(0)

    if options.readers:
        list_readers()
        sys.exit(0)

    if options.outputs:
        list_outputs()
        sys.exit(0)

    if options.enhancers:
        list_enhancers()
        sys.exit(0)

    if options.create is not None:
        generate_config(options.create)
        sys.exit(0)
   
    if options.read or options.generate:
        processor = DataProcessor(options.config)

    if options.read:
        processor.load_data()

    if options.generate:
        processor.generate_data()

